@@ -73,7 +73,7 @@ static void hfp_speaker_volume(struct ofono_call_volume *cv,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[64];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	vd->sp_volume = percent;
@@ -86,8 +86,7 @@ static void hfp_speaker_volume(struct ofono_call_volume *cv,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -101,7 +100,7 @@ static void hfp_microphone_volume(struct ofono_call_volume *cv,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[64];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	vd->mic_volume = percent;
@@ -114,8 +113,7 @@ static void hfp_microphone_volume(struct ofono_call_volume *cv,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -228,12 +226,12 @@ static struct ofono_call_volume_driver driver = {
 	.mute			= NULL,
 };
 
-void hfp_call_volume_init()
+void hfp_call_volume_init(void)
 {
 	ofono_call_volume_driver_register(&driver);
 }
 
-void hfp_call_volume_exit()
+void hfp_call_volume_exit(void)
 {
 	ofono_call_volume_driver_unregister(&driver);
 }
