@@ -338,8 +338,8 @@ static void rat_resp_cb(const GIsiMessage *msg, void *data)
 			goto error;
 	}
 
-	if (g_isi_client_send(nd->client, req, sizeof(req), NETWORK_TIMEOUT,
-				reg_status_resp_cb, cbd, NULL) != NULL)
+	if (g_isi_client_send(nd->client, req, sizeof(req),
+				reg_status_resp_cb, cbd, NULL))
 		return;
 
 error:
@@ -368,8 +368,8 @@ static void isi_registration_status(struct ofono_netreg *netreg,
 	if (nd == NULL || cbd == NULL)
 		goto error;
 
-	if (g_isi_client_send(nd->client, rat, sizeof(rat), NETWORK_TIMEOUT,
-				rat_resp_cb, cbd, NULL) != NULL)
+	if (g_isi_client_send(nd->client, rat, sizeof(rat),
+				rat_resp_cb, cbd, NULL))
 		return;
 
 error:
@@ -449,8 +449,8 @@ static void isi_current_operator(struct ofono_netreg *netreg,
 	if (cbd == NULL || nd == NULL)
 		goto error;
 
-	if (g_isi_client_send(nd->client, msg, sizeof(msg), NETWORK_TIMEOUT,
-				name_get_resp_cb, cbd, g_free) != NULL)
+	if (g_isi_client_send(nd->client, msg, sizeof(msg),
+				name_get_resp_cb, cbd, g_free))
 		return;
 
 error:
@@ -552,9 +552,9 @@ static void isi_list_operators(struct ofono_netreg *netreg,
 	if (cbd == NULL || nd == NULL)
 		goto error;
 
-	if (g_isi_client_send(nd->client, msg, sizeof(msg),
+	if (g_isi_client_send_with_timeout(nd->client, msg, sizeof(msg),
 				NETWORK_SCAN_TIMEOUT, available_resp_cb, cbd,
-				g_free) != NULL)
+				g_free))
 		return;
 
 error:
@@ -599,9 +599,9 @@ static void isi_register_auto(struct ofono_netreg *netreg,
 	if (nd == NULL || cbd == NULL)
 		goto error;
 
-	if (g_isi_client_send(nd->client, msg, sizeof(msg),
-				NETWORK_SET_TIMEOUT, set_auto_resp_cb, cbd,
-				g_free) != NULL)
+	if (g_isi_client_send_with_timeout(nd->client, msg, sizeof(msg),
+				NETWORK_SET_TIMEOUT,
+				set_auto_resp_cb, cbd, g_free))
 		return;
 
 error:
@@ -653,8 +653,9 @@ static void isi_register_manual(struct ofono_netreg *netreg,
 	if (cbd == NULL || nd == NULL)
 		goto error;
 
-	if (g_isi_client_send(nd->client, msg, sizeof(msg), NETWORK_SET_TIMEOUT,
-				set_manual_resp_cb, cbd, g_free) != NULL)
+	if (g_isi_client_send_with_timeout(nd->client, msg, sizeof(msg),
+				NETWORK_SET_TIMEOUT,
+				set_manual_resp_cb, cbd, g_free))
 		return;
 
 error:
@@ -690,7 +691,7 @@ static gboolean parse_nettime(GIsiSubBlockIter *iter,
 	struct network_time *time;
 	size_t len = sizeof(struct network_time);
 
-	if (!g_isi_sb_iter_get_struct(iter, (void **)&time, len, 0))
+	if (!g_isi_sb_iter_get_struct(iter, (void **) &time, len, 2))
 		return FALSE;
 
 	/* Value is years since last turn of century */
@@ -783,8 +784,8 @@ static void isi_strength(struct ofono_netreg *netreg,
 	if (nd == NULL || cbd == NULL)
 		goto error;
 
-	if (g_isi_client_send(nd->client, msg, sizeof(msg), NETWORK_TIMEOUT,
-				rssi_resp_cb, cbd, g_free) != NULL)
+	if (g_isi_client_send(nd->client, msg, sizeof(msg),
+				rssi_resp_cb, cbd, g_free))
 		return;
 
 error:
@@ -853,8 +854,8 @@ static struct ofono_netreg_driver driver = {
 	.name			= "isimodem",
 	.probe			= isi_netreg_probe,
 	.remove			= isi_netreg_remove,
-	.registration_status 	= isi_registration_status,
-	.current_operator 	= isi_current_operator,
+	.registration_status	= isi_registration_status,
+	.current_operator	= isi_current_operator,
 	.list_operators		= isi_list_operators,
 	.register_auto		= isi_register_auto,
 	.register_manual	= isi_register_manual,
@@ -862,12 +863,12 @@ static struct ofono_netreg_driver driver = {
 	.strength		= isi_strength,
 };
 
-void isi_netreg_init()
+void isi_netreg_init(void)
 {
 	ofono_netreg_driver_register(&driver);
 }
 
-void isi_netreg_exit()
+void isi_netreg_exit(void)
 {
 	ofono_netreg_driver_unregister(&driver);
 }
