@@ -25,6 +25,7 @@
 
 #include <errno.h>
 #include <ctype.h>
+#include <stdlib.h>
 
 #include <libudev.h>
 
@@ -54,38 +55,34 @@ static struct ofono_modem *find_modem(const char *devpath)
 	return NULL;
 }
 
-static const char *get_driver(struct udev_device *udev_device)
+static const char *get_property(struct udev_device *device,
+				char const *property_name)
 {
 	struct udev_list_entry *entry;
-	const char *driver = NULL;
 
-	entry = udev_device_get_properties_list_entry(udev_device);
+	entry = udev_device_get_properties_list_entry(device);
 	while (entry) {
 		const char *name = udev_list_entry_get_name(entry);
 
-		if (g_strcmp0(name, "OFONO_DRIVER") == 0)
-			driver = udev_list_entry_get_value(entry);
+		if (g_strcmp0(name, property_name) == 0)
+			return udev_list_entry_get_value(entry);
 
 		entry = udev_list_entry_get_next(entry);
 	}
 
-	return driver;
+	return NULL;
+}
+
+static const char *get_driver(struct udev_device *udev_device)
+{
+	return get_property(udev_device, "OFONO_DRIVER");
 }
 
 static const char *get_serial(struct udev_device *udev_device)
 {
-	struct udev_list_entry *entry;
-	const char *serial = NULL;
+	const char *serial;
 
-	entry = udev_device_get_properties_list_entry(udev_device);
-	while (entry) {
-		const char *name = udev_list_entry_get_name(entry);
-
-		if (g_strcmp0(name, "ID_SERIAL_SHORT") == 0)
-			serial = udev_list_entry_get_value(entry);
-
-		entry = udev_list_entry_get_next(entry);
-	}
+	serial = get_property(udev_device, "ID_SERIAL_SHORT");
 
 	if (serial != NULL) {
 		unsigned int i, len = strlen(serial);
@@ -146,8 +143,7 @@ static void add_mbm(struct ofono_modem *modem,
 			g_str_has_suffix(desc, "Mini-Card Network Adapter") ||
 			g_str_has_suffix(desc, "Broadband Network Adapter") ||
 			g_str_has_suffix(desc, "Minicard NetworkAdapter")) {
-		devnode = udev_device_get_property_value(udev_device,
-								"INTERFACE");
+		devnode = get_property(udev_device, "INTERFACE");
 		ofono_modem_set_string(modem, NETWORK_INTERFACE, devnode);
 	} else {
 		return;
@@ -193,8 +189,7 @@ static void add_hso(struct ofono_modem *modem,
 		else if (g_str_has_suffix(type, "Control") == TRUE)
 			ofono_modem_set_string(modem, CONTROL_PORT, devnode);
 	} else if (g_str_equal(subsystem, "net") == TRUE) {
-		devnode = udev_device_get_property_value(udev_device,
-								"INTERFACE");
+		devnode = get_property(udev_device, "INTERFACE");
 		ofono_modem_set_string(modem, NETWORK_INTERFACE, devnode);
 	} else {
 		return;
@@ -213,6 +208,7 @@ static void add_hso(struct ofono_modem *modem,
 static void add_ifx(struct ofono_modem *modem,
 					struct udev_device *udev_device)
 {
+	struct udev_list_entry *entry;
 	const char *devnode;
 
 	DBG("modem %p", modem);
@@ -220,6 +216,21 @@ static void add_ifx(struct ofono_modem *modem,
 	devnode = udev_device_get_devnode(udev_device);
 	ofono_modem_set_string(modem, "Device", devnode);
 
+	entry = udev_device_get_properties_list_entry(udev_device);
+	while (entry) {
+		const char *name = udev_list_entry_get_name(entry);
+		const char *value = udev_list_entry_get_value(entry);
+
+		if (g_str_equal(name, "OFONO_IFX_LDISC") == TRUE)
+			ofono_modem_set_string(modem, "LineDiscipline", value);
+		else if (g_str_equal(name, "OFONO_IFX_AUDIO") == TRUE)
+			ofono_modem_set_string(modem, "AudioSetting", value);
+		else if (g_str_equal(name, "OFONO_IFX_LOOPBACK") == TRUE)
+			ofono_modem_set_string(modem, "AudioLoopback", value);
+
+		entry = udev_list_entry_get_next(entry);
+	}
+
 	ofono_modem_register(modem);
 }
 
@@ -336,6 +347,9 @@ static void add_huawei(struct ofono_modem *modem,
 
 			pcui = 1;
 			ofono_modem_set_integer(modem, "PcuiRegistered", pcui);
+		} else if (g_str_equal(type, "NDIS") == TRUE) {
+			devnode = udev_device_get_devnode(udev_device);
+			ofono_modem_set_string(modem, "NDIS", devnode);
 		}
 
 		break;
@@ -345,6 +359,38 @@ static void add_huawei(struct ofono_modem *modem,
 		ofono_modem_register(modem);
 }
 
+static void add_sierra(struct ofono_modem *modem,
+					struct udev_device *udev_device)
+{
+	struct udev_list_entry *entry;
+	const char *devnode;
+	gboolean found = FALSE;
+
+	DBG("modem %p", modem);
+
+	entry = udev_device_get_properties_list_entry(udev_device);
+	while (entry) {
+		const char *name = udev_list_entry_get_name(entry);
+		const char *value = udev_list_entry_get_value(entry);
+
+		if (g_str_equal(name, "OFONO_SIERRA_TYPE") == TRUE &&
+					g_str_equal(value, "modem") == TRUE) {
+			found = TRUE;
+			break;
+		}
+
+		entry = udev_list_entry_get_next(entry);
+	}
+
+	if (found == FALSE)
+		return;
+
+	devnode = udev_device_get_devnode(udev_device);
+	ofono_modem_set_string(modem, "Device", devnode);
+
+	ofono_modem_register(modem);
+}
+
 static void add_novatel(struct ofono_modem *modem,
 					struct udev_device *udev_device)
 {
@@ -406,6 +452,106 @@ static void add_nokia(struct ofono_modem *modem,
 	}
 }
 
+static void add_isi(struct ofono_modem *modem,
+					struct udev_device *udev_device)
+{
+	const char *ifname, *type, *addr;
+
+	DBG("modem %p", modem);
+
+	if (ofono_modem_get_string(modem, "Interface"))
+		return;
+
+	addr = get_property(udev_device, "OFONO_ISI_ADDRESS");
+	if (addr != NULL)
+		ofono_modem_set_integer(modem, "Address", atoi(addr));
+
+	if (g_strcmp0(udev_device_get_subsystem(udev_device), "net") != 0)
+		return;
+
+	type = udev_device_get_sysattr_value(udev_device, "type");
+	if (g_strcmp0(type, "820") != 0)
+		return;
+
+	ifname = udev_device_get_sysname(udev_device);
+	ofono_modem_set_string(modem, "Interface", ifname);
+
+	DBG("interface %s", ifname);
+
+	ofono_modem_register(modem);
+}
+
+static void add_gobi(struct ofono_modem *modem,
+					struct udev_device *udev_device)
+{
+	struct udev_list_entry *entry;
+	const char *devnode;
+	gboolean found = FALSE;
+
+	DBG("modem %p", modem);
+
+	entry = udev_device_get_properties_list_entry(udev_device);
+	while (entry) {
+		const char *name = udev_list_entry_get_name(entry);
+		const char *value = udev_list_entry_get_value(entry);
+
+		if (g_str_equal(name, "OFONO_GOBI_TYPE") == TRUE &&
+					g_str_equal(value, "modem") == TRUE) {
+			found = TRUE;
+			break;
+		}
+
+		entry = udev_list_entry_get_next(entry);
+	}
+
+	if (found == FALSE)
+		return;
+
+	devnode = udev_device_get_devnode(udev_device);
+	ofono_modem_set_string(modem, "Device", devnode);
+
+	ofono_modem_register(modem);
+}
+
+static void add_calypso(struct ofono_modem *modem,
+					struct udev_device *udev_device)
+{
+	const char *devnode;
+
+	DBG("modem %p", modem);
+
+	devnode = udev_device_get_devnode(udev_device);
+	ofono_modem_set_string(modem, "Device", devnode);
+
+	ofono_modem_register(modem);
+}
+
+static void add_tc65(struct ofono_modem *modem,
+			struct udev_device *udev_device)
+{
+	const char *devnode;
+
+	DBG("modem %p", modem);
+
+	devnode = udev_device_get_devnode(udev_device);
+	ofono_modem_set_string(modem, "Device", devnode);
+
+	ofono_modem_register(modem);
+}
+
+static void add_nokiacdma(struct ofono_modem *modem,
+					struct udev_device *udev_device)
+{
+	const char *devnode;
+
+	DBG("modem %p", modem);
+
+	devnode = udev_device_get_devnode(udev_device);
+	ofono_modem_set_string(modem, "Device", devnode);
+
+	ofono_modem_register(modem);
+}
+
 static void add_modem(struct udev_device *udev_device)
 {
 	struct ofono_modem *modem;
@@ -482,10 +628,24 @@ done:
 		add_zte(modem, udev_device);
 	else if (g_strcmp0(driver, "huawei") == 0)
 		add_huawei(modem, udev_device);
+	else if (g_strcmp0(driver, "sierra") == 0)
+		add_sierra(modem, udev_device);
 	else if (g_strcmp0(driver, "novatel") == 0)
 		add_novatel(modem, udev_device);
 	else if (g_strcmp0(driver, "nokia") == 0)
 		add_nokia(modem, udev_device);
+	else if (g_strcmp0(driver, "isiusb") == 0)
+		add_isi(modem, udev_device);
+	else if (g_strcmp0(driver, "n900") == 0)
+		add_isi(modem, udev_device);
+	else if (g_strcmp0(driver, "gobi") == 0)
+		add_gobi(modem, udev_device);
+	else if (g_strcmp0(driver, "calypso") == 0)
+		add_calypso(modem, udev_device);
+	else if (g_strcmp0(driver, "tc65") == 0)
+		add_tc65(modem, udev_device);
+	else if (g_strcmp0(driver, "nokiacdma") == 0)
+		add_nokiacdma(modem, udev_device);
 }
 
 static gboolean devpath_remove(gpointer key, gpointer value, gpointer user_data)
@@ -508,7 +668,7 @@ static void remove_modem(struct udev_device *udev_device)
 	DBG("%s", curpath);
 
 	devpath = g_hash_table_lookup(devpath_list, curpath);
-	if (!devpath)
+	if (devpath == NULL)
 		return;
 
 	modem = find_modem(devpath);
@@ -537,6 +697,7 @@ static void enumerate_devices(struct udev *context)
 
 	udev_enumerate_add_match_subsystem(enumerate, "tty");
 	udev_enumerate_add_match_subsystem(enumerate, "net");
+	udev_enumerate_add_match_subsystem(enumerate, "hsi");
 
 	udev_enumerate_scan_devices(enumerate);
 
@@ -552,7 +713,8 @@ static void enumerate_devices(struct udev *context)
 			subsystem = udev_device_get_subsystem(device);
 
 			if (g_strcmp0(subsystem, "tty") == 0 ||
-					g_strcmp0(subsystem, "net") == 0)
+					g_strcmp0(subsystem, "net") == 0 ||
+					g_strcmp0(subsystem, "hsi") == 0)
 				add_modem(device);
 
 			udev_device_unref(device);
@@ -585,11 +747,13 @@ static gboolean udev_event(GIOChannel *channel,
 
 	if (g_str_equal(action, "add") == TRUE) {
 		if (g_strcmp0(subsystem, "tty") == 0 ||
-					g_strcmp0(subsystem, "net") == 0)
+				g_strcmp0(subsystem, "net") == 0 ||
+					g_strcmp0(subsystem, "hsi") == 0)
 			add_modem(device);
 	} else if (g_str_equal(action, "remove") == TRUE) {
 		if (g_strcmp0(subsystem, "tty") == 0 ||
-					g_strcmp0(subsystem, "net") == 0)
+				g_strcmp0(subsystem, "net") == 0 ||
+					g_strcmp0(subsystem, "hsi") == 0)
 			remove_modem(device);
 	}
 
@@ -630,7 +794,7 @@ static int udev_init(void)
 {
 	devpath_list = g_hash_table_new_full(g_str_hash, g_str_equal,
 						g_free, g_free);
-	if (!devpath_list) {
+	if (devpath_list == NULL) {
 		ofono_error("Failed to create udev path list");
 		return -ENOMEM;
 	}
@@ -653,6 +817,7 @@ static int udev_init(void)
 
 	udev_monitor_filter_add_match_subsystem_devtype(udev_mon, "tty", NULL);
 	udev_monitor_filter_add_match_subsystem_devtype(udev_mon, "net", NULL);
+	udev_monitor_filter_add_match_subsystem_devtype(udev_mon, "hsi", NULL);
 
 	udev_monitor_filter_update(udev_mon);
 
